# -*- coding: utf-8 -*-

# Copyright (c) 2006-2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPORMs.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# This module holds all the ORM classes for sqlalchemy.
# The DataManager object handles these classes.
# This module must only contain ORMs classes no other classes or functions
# as the DataManager assumes every object in here to be a ORM

# The classes below are ORMs which will be mapped on dbase tables by sqlalchemy
# and must be new style classes (derived from object).
# IMPORTANT: You must use the dbase table names as names for the ORMs.
import sqlalchemy as sqla
import sqlalchemy.orm as sqlorm
import datetime
# Hash used by the quizengine.ContentFeeder to determine which contentdb table to use
#possible quiz types are: history,math,pic,royal,sayings,text,melody and general
# we don't pass the actual tables as they are requested through the Datamanager
type2table = {'history':'game_quizhistory', 'picture':'game_quizpic', \
            'text':'game_quiztext', 'math':'game_quiztext', \
            'royal':'game_quiztext', 'sayings':'game_quiztext', \
            'melody':'game_quiztext', 'regional':'game_quizregional',\
            'personal':'game_quizpersonal'}

# ORMs for the btp_content dbase
# These MUST start with 'game_'
class game_quiztext(object): pass
class game_quizhistory(object): pass
class game_released_content(object): pass
class game_quizpic(object): pass
class game_puzzle(object): pass
class game_languages(object):
    def __init__(self, language='', lang='', rapcms_lang='', comment=''):
        self.language = language
        self.lang = lang
        self.rapcms_lang = rapcms_lang
        self.comment = comment
class game_findit(object): pass
class game_filenames(object): pass
class game_available_content(object): pass
class game_quotes(object): pass


# ORMs for the sp_users dbase
# Which attributes must be set depends if you want the ORM to be used to write
# to the table, for every column you want to write too you must set attributes.
class users(object):
    def __init__(self, login_name='', title='', first_name='', last_name='',\
                  birthdate=datetime.datetime(1900,1,1), \
                    group='', profile='', passwrd='', activities='', audio=50, dt_target='default',\
                    levelup_dlg='true'):
        self.login_name = login_name
        self.title = title
        self.first_name = first_name
        self.last_name = last_name
        self.birthdate = birthdate
        self.group = group
        self.profile = profile
        self.passwrd = passwrd
        self.activities = activities
        self.audio = audio
        self.st_target = dt_target
        self.levelup_dlg = levelup_dlg
        
class served_content(object):
    def __init__(self, user_id='', CID='', game_theme_id='', module='', start_time=None, \
                 count_served='', wrong=''):
        self.user_id = user_id
        self.CID = CID
        self.game_theme_id = game_theme_id
        self.module = module
        self.start_time = start_time
        self.count_served = count_served
        self.wrong = wrong

class group_names(object):
    def __init__(self, group_id='', group_name=''):
        self.group_id = group_id
        self.group_name = group_name

class spconf(object):
    def __init__(self, activity_name='', key='', value='', theme='', comment=''):
        self.activity_name = activity_name
        self.key = key
        self.value = value
        self.comment = comment
        self.theme = theme

class activity_options(object): 
    def __init__(self, activity, mu=None, sigma=None):
        self.activity = activity
        self.mu = mu
        self.sigma = sigma

class users_faces(object):
    def __init__(self, face_id='', user_id='', shape_diff = '', temp_diff = '', pic_name='', datetime=''):
        self.face_id = face_id
        self.user_id = user_id
        self.shape_diff = shape_diff
        self.temp_diff = temp_diff
        self.pic_name = pic_name
        self.datetime = datetime

class change_pass(object):
    def __init__(self, user='', passwrd=''):
        self.user = user
        self.passwrd = passwrd
    
class zorgenquete(object):
    def __init__(self, item='', state='', network=''):
        self.item = item
        self.state = state
        self.network = network
        
class dt_sequence(object):
    def __init__(self, fortune=0, act_name='', group='', level='', cycles='', target='', order=0):
        self.fortune = fortune
        self.act_name = act_name
        self.group = group
        self.level= level
        self.cycles = cycles
        self.target = target
        self.order = order
    
    def get_values(self):
        return {'fortune':self.fortune, 'act_name':self.act_name, 'group':self.group,\
                'level':self.level,'cycles':self.cycles,'target':self.target,'order':self.order}
    
class dt_sequence_manual(object):
    def __init__(self, user_id='', target=''):
        self.user_id = user_id
        self.target = target

class stats(object):
    def __init__(self, datetime='', activity_name='', message='', game_start_called=False, \
                 game_nextlevel_called=False, game_end_called=False, is_cop=False, \
                 user_id='', final_score='', session=''):
        self.datetime = datetime
        self.activity_name = activity_name
        self.message = message
        self.game_start_called = game_start_called
        self.game_end_called = game_end_called
        self.game_nextlevel_called = game_nextlevel_called
        self.is_cop = is_cop
        self.user_id =user_id
        self.final_score = final_score
        self.session = session
        
class stats_session(object):
    def __init__(self, datetime=''):
        self.datetime =datetime
               
# parent class for regular activities which provide mandatory attributes.
class act_parent(object):
    def __init__(self, user_id, timespend, start_time, end_time, level, score):
        self.user_id = user_id
        self.timespend = timespend
        self.start_time = start_time
        self.end_time = end_time
        self.level = level
        self.score = score

class quiz_general(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, total_questions=None, total_wrongs=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.total_wrongs = total_wrongs
        self.total_questions = total_questions

class quiz(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, total_questions=None, total_wrongs=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.total_wrongs = total_wrongs
        self.total_questions = total_questions
        
class quiz_text(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, total_questions=None, total_wrongs=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.total_wrongs = total_wrongs
        self.total_questions = total_questions
        
class quiz_melody(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, total_questions=None, total_wrongs=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.total_wrongs = total_wrongs
        self.total_questions = total_questions

class quiz_math(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, total_questions=None, total_wrongs=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.total_wrongs = total_wrongs
        self.total_questions = total_questions
        
class quiz_picture(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, total_questions=None, total_wrongs=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.total_wrongs = total_wrongs
        self.total_questions = total_questions
        
class quiz_history(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, total_questions=None, total_wrongs=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.total_wrongs = total_wrongs
        self.total_questions = total_questions
        
class quiz_royal(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, total_questions=None, total_wrongs=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.total_wrongs = total_wrongs
        self.total_questions = total_questions
        
class quiz_sayings(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, total_questions=None, total_wrongs=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.total_wrongs = total_wrongs
        self.total_questions = total_questions
        
class findit_sp(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, total_diffs=None,  wrongs=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.wrongs = wrongs
        self.total_diffs = total_diffs
        
class electro_sp(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, cards=0, knowncards=0):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.cards = cards
        self.knowncards = knowncards

class memory_sp(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, knowncards=None, cards=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.knowncards = knowncards
        self.cards = cards
        
class simon_sp(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, sounds=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.sounds = sounds
        
class puzzle(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, total_pieces=None, wrongs=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.wrongs = wrongs
        self.total_pieces = total_pieces
        
class soundmemory(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, sounds=None, knownsounds=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.sounds = sounds
        self.knownsounds = knownsounds
        
class ichanger(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, wrongs=None, extra=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.wrongs = wrongs
        self.extra = extra 
        
class fishtank(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, fish=None, clearfish=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.fish = fish
        self.clearfish = clearfish
        
class photoalbum(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        
class numbers_sp(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        
class video(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, movie=None, answer=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.movie = movie
        self.answer = answer
        
class dltr(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, activity=None, done=None, cycles=None, epoch=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.activity = activity
        self.done = done
        self.cycles = cycles
        self.epoch = epoch
        
class spinbottle(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, total=None, wrong=None, correct=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.total = total
        self.wrong = wrong
        self.correct = correct

class list_game(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, total=None, wrong=None, correct=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.total = total
        self.wrong = wrong
        self.correct = correct
 
class story(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        
class dictionary(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        
class wipe(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)

class fourrow(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None, stonesplayed=0):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)
        self.stonesplayed = stonesplayed

class flashcards(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)

class test_act(act_parent):
    def __init__(self, user_id=None, timespend=None, start_time=None, end_time=None,
                  level=None, score=None):
        act_parent.__init__(self, user_id, timespend, start_time, end_time, level, score)





# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           fallingletters.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation. A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.fallingletters.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.fallingletters")

# standard modules you probably need
import os,sys,random,string

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils
import Timer

if utils.get_locale()[0][:2] in SUPPORTEDKEYMAPS:
    LANG = utils.get_locale()[0][:2]
else:
    LANG = None
module_logger.debug("LANG set to %s" % LANG)

# containers that can be used globally to store stuff
class Misc:
    pass
    
class Letter(SPSpriteUtils.SPSprite):# we derive from this high-level class
    def __init__(self,c,x,y,step,delay):
        # create a image of the character c
        size = 48
        effect_amt = 6
        rectsizex,rectsizey = 80,80
        r,g,b = 255,0,0
        #self.image= utils.shadefade(c,size,effect_amt,(rectsizex,rectsizey),(r,g,b))
        self.image = utils.char2surf(c,size, fcol=RED, bold=True)
        SPSpriteUtils.SPSprite.__init__(self,self.image)# embed it in this class
        self.letter = c
        start = (x,y)
        self.delay = delay
        self.wait = delay
        self.rect = self.image.get_rect()
        end = (x,600)# move of the screen
        loop = 0# 0 means one time, 1 means twice
        # setup the movement of this object and get a reference to the iterator
        # which we need in the on_update method.
        self.moveit = self.set_movement(start,end,step,0,loop)
        self.connect_callback(self.callback, event_type=KEYDOWN)
        # counter that holds the number of on_update calls.
        # It is used to calculate the score value.
        # The bigger the counter the longer the object was visible
        self.visibletime = 0 
        
    def __str__(self):
        return self.letter
    
    def _get_visiblecounter(self):
        """This returns the visibletime counter devided by 30 as the counter
        is updated 30  times a second (currently SP is running at a frame rate of 30)"""
        return self.visibletime / 30.0 
    
    def on_update(self,*args):
        """This is called by group.refresh method"""
        self.visibletime += 1
        if self.wait:# extra delay, otherwise they drop still to fast 
            self.wait -= 1
            return
        else:
            self.wait = self.delay
            # we must call the iterator ourselfs because we override this method
            # from the SPSprite class
            # We override it because want to know if the movement is completed,
            # meaning user failed to type the correct character.
            # Otherwise the 'next' call is done by the SPSprite class
            status = self.moveit.next()
            if status != -1:# reached the end of the movement
                self.remove_sprite()
                # Update the missed_letters counter
                Misc.missed_letters += 1
                # update the time spend counter
                Misc.timespend += self._get_visiblecounter()
                # update the counter in the menubar
                Misc.execounter.increase_counter()

    def callback(self,*args):
        """ This is called when the sprite class update method is called.
        Remember that you must 'connect' a callback first with a call to
        self.connect_callback, see the CPSprite reference for info on possibilities
        of connecting callbacks .
        """
        try:
            c = args[1].unicode.upper()
        except ValueError:
            return
        module_logger.debug("received event: %s" % [args])
        if c == str(self).upper():
            self.remove_sprite()
            # update the time spend counter
            Misc.timespend += self._get_visiblecounter()
            # speak letter, if available
            if Misc.language:
                utils.speak_letter(c,Misc.language)
            # update the counter in the menubar
            Misc.execounter.increase_counter()
            return

class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.fallingletters.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.theme = self.SPG.get_theme()
        # alphabetsounds playing
        Misc.language = self.SPG.get_localesetting()[0]
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','FallinglettersData')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'c'))
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
    
    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.refresh()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Fallingletters")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "fallingletters"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
        _("Type the falling letters on the keyboard before they hit the ground."),
        _("In the last two levels the uppercase and lowercase are mixed but you don't have to match the case only the letter"),
        " "]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return [_("Correctness is more important than speed")]
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Alphabet/Keyboardtraining")
        
    
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 6
    
    def pre_level(self,level):
        """Mandatory method.
        Return True to call the eventloop after this method is called."""
        self.logger.debug("pre_level called with: %s" % level)
        pass
        
    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.logger.debug("nextlevel called with: %s" % level)
        # Your top blit position, this depends on the menubar position 
        if self.blit_pos[1] == 0:
            y = 10
        else:
            y = 110
        if level > 6: return False # no more then 6 levels
        # make sure we don't have a timer running
        self.stop_timer()
        # db_mapper is a Python class object that provides easy access to the dbase.
        self.db_mapper = dbmapper
        # make sure we don't have objects left in the actives group.
        self.actives.empty()
        # reset the screen to clear any crap from former levels
        self.clear_screen()
        
        # (re)set counters
        Misc.missed_letters = 0
        Misc.wrong_letters = 0
        Misc.timespend = 0
        
        # setup the letters objects
        self.letter_objects = []
        if level == 1:
            # only numbers
            chars = self.numbers_list[:]# a copy just to be sure 
            delay = 2
            step = 1
            timersleep = 3
        elif level in (2,3):
            chars = self.letters_list[:]
            delay = 4 - level
            step = 1
            timersleep = 5 - level
        elif level in (4,5):
            chars = []
            # make lower caps
            for c in self.letters_list:
                chars.append(c.lower())
            delay = 6 - level
            step = 1
            timersleep = 7 - level
        else:
            # mix upper case and lower case and numbers
            chars = self.get_mixed_list(self.letters_list[:])
            chars += self.numbers_list[:]
            delay = 1
            step = 1
            timersleep = 2
        # num_chars is used in the score calculation
        self.num_chars = len(chars)
            
        random.shuffle(chars)
        for c in chars:
            x = random.randint(10,740)
            # create a list with Letter instances
            self.letter_objects.append(Letter(c,x,y-10,step,delay))
        # Tell the core to display a exercises/done counter
        Misc.execounter = self.SPG.tellcore_display_execounter(int(self.num_chars))
        # special threaded timer object
        self.timer = Timer.Timer(delay=timersleep,func=self.drop_letter,lock=self.lock, startnow=True)

        # return True to tell the core were not done yet
        return True
        
    def start(self):
        """Mandatory method."""
        # first we setup two lists, one with numbers 0-9 and one with letters A-Z
        # we convert them to images in next_level
        self.letters_list = list(string.lowercase)
        if LANG:
            newlist = []
            for char in self.letters_list:
                newlist.append(utils.map_keys(LANG, char.upper()))
            self.letters_list = newlist
        self.numbers_list = list(string.digits)
        # we will also use timers to 'launch' the letters
        self.timer = None # will hold the timer object
        # because we will use timers we must pass them the SPC lock object.
        # The lock is used to lock every thread when a dialog is shown and
        # everything should 'freeze'
        self.lock = self.SPG.get_thread_lock()
    
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        self.timer.start()
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        # timespend is not used, we keeping time ourselfs
        # As this method is called by the core when a level is finished
        # we also use it to store the counters
        self.db_mapper.insert('missedletters',Misc.missed_letters)
        self.db_mapper.insert('wrongletters',Misc.wrong_letters)
        self.db_mapper.insert('totaltime',Misc.timespend)
        # add wrongletters to dbase, TODO: add column to table
        # calculate score value
        # For every missed letter we add 40 seconds to timespend.
        # For every wrong letter we add 10 seconds to timespend.
        # An educated guess (tested with some children) would be the following
        # scores. The average seconds are calculated by dividing the timespend
        # by 26, the number of characters shown. 
        # 1 second average == 10
        # 13 second average == 7
        # 23 second average == 5.5
        # >43 == 1
        seconds = Misc.timespend
        seconds += (40.0 * Misc.missed_letters)
        seconds += (10.0 * Misc.wrong_letters)
        score = max(0.2, 10 - ((((seconds/26.0)*0.7) ** 0.6)-0.8))
        if score > 10:
            score == 10
        self.logger.debug('missed_letters %s' % Misc.missed_letters)
        self.logger.debug('wrong_letters %s' % Misc.wrong_letters)
        self.logger.debug('total time %s' % seconds)
        self.logger.debug('score %s' % score)
        return score
    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass

    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        myevent = None
        if not self.letter_objects and not self.actives.sprites():
            # no more sprites left so the level is done.
            # we call the SPGoodies observer to notify the core the level
            # is ended and we want to store the collected data
            self.SPG.tellcore_level_end(store_db=True)
        for event in events:
            if event.type is KEYDOWN:
                # no need to query all sprites with all the events
                # we only pass it an event if it's an event were intrested in.
                myevent = event
                # first we check if the event key is part of the active sprites
                # if not we increase the Misc.wrong_letters counter.
                if event.unicode.upper() not in [x.letter.upper() for x in self.actives.sprites()]:
                    Misc.wrong_letters += 1
                    break
        # move all images
        self.actives.refresh(myevent)
        return 

    def drop_letter(self):
        """Called by the timer, and responsible for the letter dropping.
        This will get a letter from the letter_objects list and place it in
        the Sprite actives group, and thereby it will be blitted on the screen."""
        try:
            obj = self.letter_objects.pop()
        except IndexError:
            return 0
        self.actives.add(obj)
        return 1        
    
    def get_mixed_list(self,charlist):
        # now we shuffle and split the list in two, one halve will
        # become lower and the other upper.
        random.shuffle(charlist)
        lowerlist = charlist[13:]
        upperlist = [x.lower() for x in charlist[:13]]
        charlist = lowerlist+upperlist
        random.shuffle(charlist)
        return charlist
          



# This module comes from the OLPC project and is licensed under the BSD license.
# The changes are minimal and only done to let it work outside Sugar.

# Below is the original COPYING file.
##* Copyright (c) 2007, One Laptop Per Child.
##* All rights reserved.
##*
##* Redistribution and use in source and binary forms, with or without
##* modification, are permitted provided that the following conditions are met:
##*     * Redistributions of source code must retain the above copyright
##*       notice, this list of conditions and the following disclaimer.
##*     * Redistributions in binary form must reproduce the above copyright
##*       notice, this list of conditions and the following disclaimer in the
##*       documentation and/or other materials provided with the distribution.
##*     * Neither the name of One Laptop Per Child nor the
##*       names of its contributors may be used to endorse or promote products
##*       derived from this software without specific prior written permission.
##*
##* THIS SOFTWARE IS PROVIDED BY ONE LAPTOP PER CHILD ``AS IS'' AND ANY
##* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
##* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
##* DISCLAIMED. IN NO EVENT SHALL ONE LAPTOP PER CHILD BE LIABLE FOR ANY
##* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
##* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
##* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
##* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
##* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
##* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Utility functions for cairo-specific operations

USE_BASE_ARRAY -- if False (default), uses numpy arrays,
    currently this is the only version that works on 32-bit
    machines.
"""
import pygame, struct, logging
big_endian = struct.pack( '=i', 1 ) == struct.pack( '>i', 1 )

log = logging.getLogger( 'schoolsplay._cairoimage' )
##log.setLevel( logging.DEBUG )

USE_BASE_ARRAY = False

def newContext( width, height ):
    """Create a new render-to-image context
    
    width, height -- pixel dimensions to be rendered
    
    Produces an ARGB format Cairo ImageSurface for 
    rendering your data into using rsvg, Cairo or Pango.
    
    returns (ImageSurface, CairoContext) for rendering
    """
    import cairo
    csrf = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
    context = cairo.Context (csrf)
    #log.info( 'Format (expect: %s): %s', cairo.FORMAT_ARGB32, csrf.get_format())
    return csrf, context

def mangle_color(color):
    """Mange a colour depending on endian-ness, and swap-necessity
    
    Converts a 3 or 4 int (or float) value in the range 0-255 into a 
    4-float value in the range 0.0-1.0
    """
    r,g,b = color[:3]
    if len(color) > 3:
        a = color[3]
    else:
        a = 255.0
    return map(_fixColorBase, (r,g,b,a) )

def _fixColorBase( v ):
    """Return a properly clamped colour in floating-point space"""
    return max((0,min((v,255.0))))/255.0

def asImage( csrf ):
    """Get the pixels in csrf as a Pygame image
    
    Note that Pygame 1.7.1 on (Gentoo Linux) AMD64 is incorrectly 
    calculating the required size ARGB images, so this code will *not* work 
    on that  platform with that version of the library.  Pygame-ctypes 
    does work correctly there.
    
    Note also that Pygame 1.7.1 is showing a strange colour rotation 
    bug on 32-bit platforms, such that ARGB mode cannot be used for 
    images there.  Instead we have to do an expensive bit-shift operation
    to produce an RGBA image from the ARGB native Cairo format.
    
    Will raise a ValueError if passed a Null image (i.e. dimension of 0)
    
    returns Pygame.Surface (image) with convert_alpha() called for it.
    """
    # Create and return a new Pygame Image derived from the Cairo Surface
    format = 'ARGB'
    if hasattr(csrf,'get_data'):
        # more recent API, native-format, but have to (potentially) convert the format...
        #log.debug( 'Native-mode api (get_data)' )
        data = csrf.get_data()
        if not big_endian:
            # we use array here because it's considerably lighter-weight
            # to import than the numpy module
            #log.debug( 'Not big-endian, byte-swapping array' )
            if USE_BASE_ARRAY:
                import array
                a = array.array( 'I' )
                a.fromstring( data )
                a.byteswap()
                data = a.tostring()
            else:
                import numpy
                n = numpy.fromstring( data, dtype='I' )
                n =  ((n & 0xff000000) >> 24  ) | ((n & 0x00ffffff) << 8 )
                n = n.byteswap()
                data = n.tostring()
                format = 'RGBA'
        else:
            #log.debug( 'Big-endian, array unchanged' )
            data = str(data) # there's one copy
    else:
        # older api, not native, but we know what it is...
        #log.debug( 'Non-native mode api, explicitly RGBA' )
        data = csrf.get_data_as_rgba()
        data = str(data) # there's one copy
        format = 'RGBA'
    width, height = csrf.get_width(),csrf.get_height()

    try:
        #log.info( 'Format = %s', format )
        return pygame.image.fromstring(
            data, 
            (width,height), 
            format
        ) # there's the next
    except ValueError, err:
        err.args += (len(data), (width,height), width*height*4,format )
        raise

if __name__ == "__main__":
    import unittest
    logging.basicConfig()
    class Tests( unittest.TestCase ):
        def test_colours( self ):
            """Test that colours are correctly translated
            
            If we draw a given colour in cairo, we want the same
            colour to show up in Pygame, let's test that...
            """
            for sourceColour in [
                (255,0,0, 255),
                (0,255,0, 255),
                (0,0,255, 255),
                (255,255,0, 255),
                (0,255,255,255),
                (255,0,255,255),
            ]:
                csrf,cctx = newContext( 1,1 )
                background = mangle_color( sourceColour )
                cctx.set_source_rgba(*background)
                cctx.paint()
                img = asImage( csrf )
                colour = img.get_at( (0,0))
                assert colour == sourceColour, (sourceColour,mangle_color(sourceColour),colour)
    unittest.main()




# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           birthday.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils
import SPWidgets
import glob

class Animation(SPSpriteUtils.SPSprite):
    def __init__(self, animationlist, pos=(300, 200)):
        self.part_index = 0
        self.index = 0
        self.max_index = len(animationlist) - 1
        self.animationlist = animationlist
        self.wait = 5
        self.counter = 0
        SPSpriteUtils.SPSprite.__init__(self, self.animationlist[0])
        self.moveto(pos)
    
    def on_update(self,*args):
        """This is called by group.refresh method"""
        if self.counter < self.wait: 
            self.counter += 1
            return
        else:
            self.counter = 0
            self.erase_sprite()
            if self.index < self.max_index:
                self.index += 1
            else:
                self.index = 0
            self.image = self.animationlist[self.index]
        self.display_sprite()

class birthday:
    def __init__(self, screen, back, name, age):
        self.screen = screen
        self.back = back
        self.actives = SPSpriteUtils.SPInit(self.screen, self.back)
        but = SPWidgets.SimpleButtonDynamic(_("Close"), (350, 500), fsize=24, data='close')
        but.set_use_current_background(True)
        but.display_sprite()
        self.actives.add(but)
        lbl = SPWidgets.Label(_("Congratulations %s, you are %s year old today.") % (name, age), \
                              (100, 120), fgcol=ORANGERED, transparent=True).display_sprite()
        self.datadir = os.path.join(ACTIVITYDATADIR, 'SPData', 'themes', 'braintrainer', 'birthday')
        # we only use cake for now
        self.animations_path = os.path.join(self.datadir, 'party', '*')
        self.animationlist = []
        for file in glob.glob(self.animations_path):
            self.animationlist.append(utils.load_image(file))
        self.animation = Animation(self.animationlist)
                    
    def run(self):
        runloop = 100
        
        while runloop:
            runloop -= 1
            pygame.time.wait(100)
            pygame.event.pump()
            events = pygame.event.get()
            for event in events:
                if event.type is KEYDOWN:
                    if event.key == K_ESCAPE:
                        runloop = 0
                if self.actives.update(event):
                    runloop = 0
            self.animation.on_update()
            
        

if __name__ == '__main__':
    
    import __builtin__
    __builtin__.__dict__['_'] = lambda x:x
    
    import SPLogging
    SPLogging.set_level('debug')
    SPLogging.start()
    
    pygame.init()
    
    from SPSpriteUtils import SPInit
    
    def cbf(sprite, event, data=''):
        print 'cb called with sprite %s, event %s and data %s' % (sprite, event, data)
        print 'sprite name: %s' % sprite.get_name()
        print 'data is %s' % data
    
    scr = pygame.display.set_mode((800, 600))
    scr.fill(GREY90)
    pygame.display.flip()
    back = scr.convert()
    actives = SPInit(scr, back)
    
    SPWidgets.Init('braintrainer')
    
    b = birthday(scr, back)
    b.run()
    




# -*- coding: utf-8 -*-

# Copyright (c) 2011 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
# Code is taken from 
#   http://inventwithpython.com/blog/2011/06/10/new-game-source-code-four-in-a-row
# 
# Graphical Four-In-A-Row (a Connect Four clone)
# http://inventwithpython.com/blog
# By Al Sweigart al@inventwithpython.com
#
#           fourrow.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation. A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.fourrow.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.fourrow")

# standard modules you probably need
import os,sys
import random
import copy
import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils

from SPWidgets import Label, ButtonDynamic

BOARDWIDTH = 7  # how many spaces wide the board is
BOARDHEIGHT = 6 # how many spaces tall the board is
PILSIZE = 70
class Pil(SPSpriteUtils.SPSprite):
    def __init__(self, image, pos,id):
        SPSpriteUtils.SPSprite.__init__(self,image,id)
        self.moveto(pos)
        
class Tile(SPSpriteUtils.SPSprite):
    def __init__(self, image, pos):
        SPSpriteUtils.SPSprite.__init__(self,image,id)
        self.moveto(pos)
        
class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.fourrow.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.lang = self.SPG.get_localesetting()[0][:2]
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.windowSurf = self.screen
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','FourrowData')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'fourrow.rc'))
        self.logger.debug("rchash: " + str(self.rchash))
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
                
    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        #self.actives.refresh()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Fourrow")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "fourrow"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
        _("Try to get four in a row in any possible direction."),
        " ", 
        _("You can vertically drop a red pill in a desired column by pressing that exact column."),
        _("All pills, both red and yellow, stack on each other."), 
        _("To win the game you'll need four red pills lined up after each other horizontally, vertically or diagonal."), 
        _("Tip: Try creating multiple winning scenario's."), 
        " "]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return []
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Puzzle")
    def stop(self):
        """Mandatory method, will be called by the core when this activities is destroyed."""
        try:
            self.winSnd.stop()
            self.lossSnd.stop()
        except:
            pass
            
    def start(self):
        """Mandatory method."""
    
        img = utils.load_image(os.path.join(self.my_datadir,'4row_red.png'))
        self.spacesize = PILSIZE
        self.RedImg = pygame.transform.smoothscale(img, (self.spacesize, self.spacesize))
        
        img = utils.load_image(os.path.join(self.my_datadir,'4row_black.png'))
        self.BlackImg = pygame.transform.smoothscale(img, (self.spacesize, self.spacesize))
        
        img = utils.load_image(os.path.join(self.my_datadir,'4row_board.png'))
        img = pygame.transform.smoothscale(img, (self.spacesize, self.spacesize))
        self.boardImg = img
        
        self.gameClock = pygame.time.Clock()# tobe removed
        self.draggingToken = False
        self.tokenx, self.tokeny = None, None
        
        row = self.SPG.get_current_user_dbrow()
        user_id = self.SPG.get_current_user_id()
        UserImg = None
        offset = 0
        if user_id > 1:# demo is 1
            p = os.path.join(HOMEDIR, 'braintrainer', 'faces', 'big', '%s.png' % user_id)
            if os.path.exists(p):
                UserImg = utils.aspect_scale(utils.load_image(p), (125,125))
                offset = 12
        if not UserImg:
            st = utils.char2surf(row.first_name, 20, fcol=BLACK)
            sn =  utils.char2surf(row.last_name, 20, fcol=BLACK)
            surf = pygame.Surface((150,80), SRCALPHA, 32)
            surf.blit(st,(8,4))
            surf.blit(sn, (8,30))  
            UserImg = surf
        self.UserImg = SPSpriteUtils.SPSprite(UserImg)
        surf = UserImg.copy()
        pygame.draw.rect(surf, GREEN, UserImg.get_rect(), 10)
        self.TurnImgUser = SPSpriteUtils.SPSprite(surf)
        
        AIimg = utils.load_image(os.path.join(self.my_datadir,'computer.png'))
        self.AIimg = SPSpriteUtils.SPSprite(AIimg)
        surf = AIimg.copy()
        pygame.draw.rect(surf, GREEN, AIimg.get_rect(), 10)
        self.TurnAIimg = SPSpriteUtils.SPSprite(surf)
        
        self.WinnerImg = utils.load_image(os.path.join(self.my_datadir,'winner.png'))
        
        self.numbershash = {}
        for i in range(1, BOARDWIDTH+1):
            self.numbershash[i] = utils.load_image(os.path.join(self.my_datadir,'%s.png' % i))
        
        # sounds
        
        self.winSnd = utils.load_music(os.path.join(self.my_datadir,'won.ogg'))
        self.lossSnd = utils.load_music(os.path.join(self.my_datadir,'loss.ogg'))             
               
        # various constants
        self.FPS = 30 # frames per second to update the screen
        WINDOWWIDTH = 800 # width of the program's window, in pixels
        WINDOWHEIGHT = 600 # height in pixels
        
        self.XMARGIN = int((WINDOWWIDTH - BOARDWIDTH * self.spacesize) / 2) # approx. number of pixels on the left or right side of the board to the edge of the window
        self.YMARGIN = int((WINDOWHEIGHT - BOARDHEIGHT * self.spacesize) / 2 + 80) # approx. number of pixels above or below board
        
        self.REDpos = (self.XMARGIN - self.spacesize * 2, self.YMARGIN + (BOARDHEIGHT - 1) * self.spacesize)
        self.BLACKpos = (self.XMARGIN + BOARDWIDTH * self.spacesize + self.spacesize, self.YMARGIN + (BOARDHEIGHT - 1) * self.spacesize)
        self.UserImgpos = (2 + offset, 380)
        self.UserImg.moveto(self.UserImgpos)
        self.TurnImgUser.moveto(self.UserImgpos)
        self.AIimgpos = (660, 380)
        self.AIimg.moveto(self.AIimgpos)
        self.TurnAIimg.moveto(self.AIimgpos)
        self.winner = 'tie'
        self.difficulty = 1# never more then 2, system will freeze
        self.AreWeDT = False
    
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 6
    
    def pre_level(self,level):
        """Mandatory method.
        Return True to call the eventloop after this method is called."""
        self.logger.debug("pre_level called with: %s" % level)
        pass
        
    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.logger.debug("nextlevel called with: %s" % level)
        
        self.isFirstGame = True
        self.level = level
        
        self.next_exercise()
        return True
        
    def next_exercise(self, *args):
        # build board
        self.clear_screen()
        self.actives.empty()
        self.Tiles = {} 
        self.old_pil_rect = None
        # draw board and the tokens
        for x in range(BOARDWIDTH):
            posx = self.XMARGIN + (x * self.spacesize)
            #palceholder for special trans numbers
            self.screen.blit(self.numbershash[x+1], (posx, self.YMARGIN))
            for y in range(BOARDHEIGHT):
                posy = self.YMARGIN + (y * self.spacesize)
                tile = Tile(self.boardImg, (posx, posy))
                tile.connect_callback(self._cbf_Tile, MOUSEBUTTONDOWN, (posx, posy))
                if not self.Tiles.has_key(posx):
                    self.Tiles[posx] = []
                self.Tiles[posx].append(tile)
                tile.display_sprite()
                self.actives.add(tile)
        self.backgr.blit(self.screen,self.blit_pos, self.screenclip)
        self.UserImg.display_sprite()
        self.AIimg.display_sprite()
#        self.screen.blit(self.RedPil, self.REDpos)
#        self.screen.blit(self.BlackPil, self.BLACKpos)
        Pil(self.RedImg, self.REDpos,'red').display_sprite()
        Pil(self.BlackImg, self.BLACKpos, 'black').display_sprite()
        
        pygame.display.update()
        
        if self.isFirstGame:
            self.turn = 'computer'
            self.isFirstGame = False
        else:
            self.turn = random.choice(['computer', 'human'])
        if self.turn == 'computer':
            self.AIimg.erase_sprite()
            self.TurnAIimg.display_sprite()
        else:
            self.UserImg.erase_sprite()
            self.TurnImgUser.display_sprite()
        self.firstmove = True
        self.mainBoard = self.getNewBoard()
        
    def _cbf_Tile(self, event, widget, data):
        if self.turn != 'human':
            return
        self.logger.debug("human turn")
        x = data[0][0]
        column = int((x - self.XMARGIN) / self.spacesize)
        if self.isValidMove(self.mainBoard, column):
            self.animateDroppingToken(self.mainBoard, column, 'red')
            self.mainBoard[column][self.getLowestFreeSpace(self.mainBoard, column)] = 'red'
            self.old_pil_rect = None
            if self.isWinner(self.mainBoard, 'red'):
                self.winner = 'user'
            self.turn = 'computer'
            self.TurnImgUser.erase_sprite()
            self.UserImg.display_sprite()
            self.TurnAIimg.display_sprite()
        return -1
                            
    def makeMove(self, board, player, column):
        lowest = self.getLowestFreeSpace(board, column)
        if lowest != -1:
            board[column][lowest] = player
    
    def getNewBoard(self):
        board = []
        for x in range(BOARDWIDTH):
            board.append([None] * BOARDHEIGHT)
        return board
        
    def animateDroppingToken(self, board, column, color):
        x = self.XMARGIN + column * self.spacesize
        y = self.YMARGIN - self.spacesize
        speed = 10
        dirty = []
        lowestFreeSpace = self.getLowestFreeSpace(board, column)
        RedPil = Pil(self.RedImg, (x,y),'red')
        BlackPil = Pil(self.BlackImg, (x,y), 'black')
        while True:
            y += int(speed)
            if (y - self.YMARGIN - speed) / self.spacesize >= lowestFreeSpace:
                if self.Tiles[x]:
                    self.Tiles[x].pop()
                return
#            if self.old_pil_rect:
#                self.screen.blit(self.backgr, self.old_pil_rect.topleft, self.old_pil_rect)
            for t in self.Tiles[x]:
                self.screen.blit(self.backgr,t.rect, t.rect)
#            pygame.display.update(dirty)
            if color == 'black':
                BlackPil.undraw_sprite()
                BlackPil.draw_sprite((x,y))
#                r= self.screen.blit(self.BlackPil, (x,y))
            else:
                RedPil.undraw_sprite()
                RedPil.draw_sprite((x,y))
#                r = self.screen.blit(self.RedPil, (x,y))
#            self.old_pil_rect = r
#            pygame.display.update(r)
            if self.Tiles.has_key(x):
                dirty = []
                for t in self.Tiles[x]:
                    self.screen.blit(t.image,t.rect)
#                pygame.display.update(dirty)
            pygame.display.flip()
            self.gameClock.tick(self.FPS)
            
        
    def getComputerMove(self, board):
        if self.isBoardFull(board):
            self._show_winner()
        if self.firstmove and self.level > 2:
            # Added intelligent, we check if we can place in the middle, even when the user
            # placed one stone, at least we will block his attempt
            # we disable this for the first two levels
            self.firstmove = False
            cl =[]
            if self.isValidMove(board, 3):
                cl.append(3)
            if self.isValidMove(board, 4):
                cl.append(4)
            if cl:
                return random.choice(cl)
        # make lower levels easier
        if self.level == 1:
            return self.getRandomMove(0)
        elif self.level == 2:
            return self.getRandomMove(4)
        elif self.level in (3,4):
            self.difficulty = 1
            return self.getRandomMove(7)
        elif self.level == 5:
            self.difficulty = 1
            return self.getRandomMove(-1)
        else:
            self.difficulty = 2
            return self.getRandomMove(-1)
    
    def getRandomMove(self, level):
        """The higher the level, max 7, the less random it is.
        ex. 5 means a chance of 1 out of 5 to get a random move.
        -1 means no random move"""
        # first we determine which moves are legal
        legalmoves = [i for i in range(BOARDWIDTH) if self.isValidMove(self.mainBoard, i)]
        if level == 0 or level != -1 and random.randrange(level) == 0:# random move
            return random.choice(legalmoves)
        
        potentialMoves = self.getPotentialMoves(self.mainBoard, 'black', self.difficulty)
        bestMoveScore = max([potentialMoves[i] for i in range(BOARDWIDTH) if self.isValidMove(self.mainBoard, i)])
        bestMoves = []
        for i in range(len(potentialMoves)):
            if potentialMoves[i] == bestMoveScore and self.mainBoard[i][0] == None:
                bestMoves.append(i)
        return random.choice(bestMoves)
        
    def getPotentialMoves(self, board, playerTile, lookAhead):
        if lookAhead == 0 or self.level ==1:
            return [0] * BOARDWIDTH
    
        potentialMoves = []
    
        if playerTile == 'red':
            enemyTile = 'black'
        else:
            enemyTile = 'red'
    
        # Returns (best move, average condition of this state)
        if self.isBoardFull(board):
            return [0] * BOARDWIDTH
             
        # Figure out the best move to make.
        potentialMoves = [0] * BOARDWIDTH
        for playerMove in range(BOARDWIDTH):
            dupeBoard = copy.deepcopy(board)
            if not self.isValidMove(dupeBoard, playerMove):
                continue
            self.makeMove(dupeBoard, playerTile, playerMove)
            if self.isWinner(dupeBoard, playerTile):
                potentialMoves[playerMove] = 1
                break
            else:
                # do other player's moves and determine best one
                if self.isBoardFull(dupeBoard):
                    potentialMoves[playerMove] = 0
                else:
                    for enemyMove in range(BOARDWIDTH):
                        dupeBoard2 = copy.deepcopy(dupeBoard)
                        
                        if not self.isValidMove(dupeBoard2, enemyMove):
                            continue
                        self.makeMove(dupeBoard2, enemyTile, enemyMove)
                        if self.isWinner(dupeBoard2, enemyTile):
                            potentialMoves[playerMove] = -1
                            break
                        else:
                            results = self.getPotentialMoves(dupeBoard2, playerTile, lookAhead - 1)
                            potentialMoves[playerMove] += (sum(results) / BOARDWIDTH) / BOARDWIDTH                   
        return potentialMoves
    
    
    def getLowestFreeSpace(self, board, column):
        for y in range(BOARDHEIGHT-1, -1, -1):
            if board[column][y] == None:
                return y
        return -1
    
    def isValidMove(self, board, move):
        if move < 0 or move >= (BOARDWIDTH):
            return False
        if board[move][0] != None:
            return False
        return True
    
    
    def isBoardFull(self, board):
        for x in range(BOARDWIDTH):
            for y in range(BOARDHEIGHT):
                if board[x][y] == None:
                    return False
        return True
    
    
    def isWinner(self, board, tile):
        # check horizontal spaces
        for y in range(BOARDHEIGHT):
            for x in range(BOARDWIDTH - 3):
                if board[x][y] == tile and board[x+1][y] == tile and board[x+2][y] == tile and board[x+3][y] == tile:
                    return True
    
        # check vertical spaces
        for x in range(BOARDWIDTH):
            for y in range(BOARDHEIGHT - 3):
                if board[x][y] == tile and board[x][y+1] == tile and board[x][y+2] == tile and board[x][y+3] == tile:
                    return True
    
        # check / diagonal spaces
        for x in range(BOARDWIDTH - 3):
            for y in range(3, BOARDHEIGHT):
                if board[x][y] == tile and board[x+1][y-1] == tile and board[x+2][y-2] == tile and board[x+3][y-3] == tile:
                    return True
    
        # check \ diagonal spaces
        for x in range(BOARDWIDTH - 3):
            for y in range(BOARDHEIGHT - 3):
                if board[x][y] == tile and board[x+1][y+1] == tile and board[x+2][y+2] == tile and board[x+3][y+3] == tile:
                    return True
        return False
    
    def _show_winner(self):
        self.turn = None
        if self.winner == 'tie':
            self.TurnAIimg.erase_sprite()
            self.AIimg.display_sprite()
            self.TurnImgUser.erase_sprite()
            self.UserImg.display_sprite()
            pos = (self.UserImgpos[0], self.UserImgpos[1] - 50)
            SPSpriteUtils.SPSprite(self.WinnerImg.copy()).display_sprite((pos[0], pos[1] - 150))
            pos = (self.AIimgpos[0], self.AIimgpos[1] - 50)
            SPSpriteUtils.SPSprite(self.WinnerImg.copy()).display_sprite((pos[0], pos[1] - 150))
        else:
            self.TurnAIimg.erase_sprite()
            self.AIimg.display_sprite()
            self.TurnImgUser.erase_sprite()
            self.UserImg.display_sprite()
            pos = {'user':(self.UserImgpos[0], self.UserImgpos[1] - 50), \
                   'computer':(self.AIimgpos[0], self.AIimgpos[1] - 50)}[self.winner]
            SPSpriteUtils.SPSprite(self.WinnerImg).display_sprite((pos[0], pos[1] - 150))
            if self.winner == 'user':
                self.scoredisplay.increase_score(100)
                self.winSnd.play()
            elif self.winner =='tie':
                self.scoredisplay.increase_score(50)
            else:
                self.lossSnd.play()
        self.turn = 'restart'# this will cause the eventloop to call _restart
        
    def _restart(self):
        self.actives.empty()
        if self.winner == 'user':
            txt = _("Congratulations, try again ?")
        else:
            txt = _("Pity, try again ?")
        but = ButtonDynamic(txt, (250,110), fgcol=WHITE)
        but.connect_callback(self.next_exercise, MOUSEBUTTONUP)
        but.display_sprite()
        self.actives.add(but)
        
    def dailytraining_pre_level(self, level):
        """Mandatory method"""
        pass
        
    def dailytraining_next_level(self, level, dbmapper):
        """Mandatory method.
        This should handle the DT logic"""
        self.isFirstGame = True
        self.level = level
        self.AreWeDT = True
        self.next_exercise()
        return True
        
        
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        pass 
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        if self.winner == 'user':
            return 100
        else:
            return 10
    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass

    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        if not self.turn:
            for event in events:
                if event.type in (MOUSEBUTTONUP, MOUSEMOTION):
                    self.actives.update(event)
        elif self.isBoardFull(self.mainBoard):
            self.winner = 'tie'
            self._show_winner()
            
        if self.turn == 'restart':
            self.turn = None
            if self.AreWeDT:
                self.SPG.tellcore_level_end(level=self.level)
            else:
                self._restart()
            
        if self.turn == 'human':
            for event in events:
                if event.type is MOUSEBUTTONDOWN:
                    self.actives.update(event)
                                    
            if self.isWinner(self.mainBoard, 'red'):
                self.winner = 'user'
                self._show_winner()
            elif self.isBoardFull(self.mainBoard):
                self.winner = 'tie'
                self._show_winner()
                
        if self.turn == 'computer':
            self.logger.debug("computer turn")
            # computer move runs in a loop of it's own
            column = self.getComputerMove(self.mainBoard)            
            self.animateDroppingToken(self.mainBoard, column, 'black')
            self.makeMove(self.mainBoard, 'black', column)
            if self.isWinner(self.mainBoard, 'black'):
                self.winner = 'computer'
                self._show_winner()
            elif self.isBoardFull(self.mainBoard):
                self.winner = 'tie'
                self._show_winner()
            else:
                self.old_pil_rect = None
                self.turn = 'human'
                self.TurnAIimg.erase_sprite()
                self.AIimg.display_sprite()
                self.UserImg.erase_sprite()
                self.TurnImgUser.display_sprite()
        
        return 
        



# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           quiz.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.quiz_general.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.quiz_knowledge")

# standard modules you probably need
import os,sys

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils
from SPWidgets import TransPrevNextButton

try:
    from xml.etree.ElementTree import ElementTree
except ImportError:
    # try the python2.4 way 
    from elementtree.ElementTree import ElementTree

def parse_xml(xml):
    """Parses the whole xml tree into a hash with lists. Each list contains hashes
    with the elelements from a 'activity' element.
    """  
    logger = logging.getLogger("childsplay.quiz_general.parse_xml")
    logger.debug("Starting to parse: %s" % xml)

    tree = ElementTree()
    tree.parse(xml)
    xml = {}
    # here we start the parsing
    acts = tree.findall('activity')
    logger.debug("found %s activities in total" % len(acts))
    acthash = utils.OrderedDict()
    for act in acts:
        e = act.find('cycles')
        acthash[act.get('name')] = {'cycles':int(e.text)}
    logger.debug("xml hash:%s" % acthash)
    return acthash

class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
    """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.quiz_general.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.lang = self.SPG.get_localesetting()[0][:2]
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.logo_pos = (600, 200 + self.blit_pos[1])
        self.theme = self.SPG.get_theme()
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','Quiz_generalData')
        self.quiz_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','Quizengine_Data')
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'quiz_general.rc'))
        self.rchash['theme'] = self.theme
        self.rchash['lang'] = self.lang
        self.logger.debug("found rc: %s" % self.rchash)
        p = os.path.join(self.CPdatadir,'good_%s.png' % self.lang)
        if not os.path.exists(p):
            p = os.path.join(self.CPdatadir,'thumbs.png')
        self.ThumbsUp = SPSpriteUtils.MySprite(utils.load_image(p))
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
          
        self.num_questions = int(self.rchash[self.theme]['questions'])
        self.AreWeDT = False
        self.sessionresults = utils.OrderedDict()
        self.sessionresults_raw = utils.OrderedDict()
        self.old_level = None
        self.sessionid = time.time()
        self.act_score = 0
        self.act_cycles = 1
        self.act_mean = 0
        self.levelup = 0
        
    def _set_up(self, *args):
        """called by the core after this module is constructed."""
        xmlpath = os.path.join(self.my_datadir, 'general_knowledge.xml')
        if not os.path.exists(xmlpath):
            raise utils.MyError, _("xml file %s is missing, this shouldn't happen, contact the %s developers" % (xmlpath, self.theme))    
        self.actdatahash = parse_xml(xmlpath)# returns a ordereddict object
        self.actdata = self.actdatahash.items()
                
    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.refresh()
        
    def get_helptitle(self):
        """Mandatory method"""
        return "dltr"
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "quiz_general"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
                _("Answer the quiz questions."), 
        " ",
        _("At the top of the screen, you will see a question about general knowledge."),
        _("Below it are a choice of possible answers.\nTouch the correct answer."), 
        " "]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty list"""
        return []
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Miscellaneous")
        
    
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 6
    
    def _cbf(self, *args):
        return args
    
    def pre_level(self,level):
        """Mandatory method.
        Return True to call the eventloop after this method is called."""
        
    def start(self):
        """Mandatory method."""
        self.levelup = 0
        try:
            self.currentactname, self.currentactdata = self.actdata.pop(0)
        except IndexError:
            self.logger.debug("No more activities to run")
            self._set_up()
            if self.act_mean >= 7.0:
                self.levelup = 1
            else:
                self.levelup = 0
            self.currentactname, self.currentactdata = self.actdata.pop(0)
        self.logger.debug("activity %s is next." % self.currentactname)
                    
    def act_stopped(self, activity):
        """called by the core when an act has finished the levels."""
        self.logger.debug("act_stopped called")
    
    def act_next_level_stopped(self, result):
        """called by the core when the act finished a level.
        This is also called after the last level of act, just before the core calls
        act_stopped."""
        self.logger.debug("act_next_level_stopped called with: %s" % result)
        if not result:
            # it's called with None because we are also a stopping act, weird stuff :-)
            return
        self.act_score = float(self.act_score + result)
        self.act_mean = self.act_score / self.act_cycles
        self.act_cycles += 1
        self.logger.debug("activity mean score now %s" % self.act_mean)
    
    def act_hit_levelindicator(self, level):
        """Called by the core when the levelindicator is hit by the user to change
        the level. (If we enabled the levelindicator of course)
        We use this to determine if we switch acts when act_stopped is called or that
        we must restart with all acts from the new level."""
        self.logger.debug("act_hit_levelindicator called with: %s" % level)
        self._set_up()
        self.oldlevel = level
        self.level = level
        self.act_score = 0
        self.act_mean = 0
        self.cycles = 1
        
    def dt_restarting(self):
        """Called just before the core restarts the DT"""
        self.old_level = self.level

    def get_actdata(self):
        """Called by the core"""
        return self.currentactdata
    
    def next_activity(self):
        self.logger.debug("next_activity called")
        cycles = self.currentactdata['cycles']
        if self.old_level:
            level = self.old_level
            self.old_level = None
        else:
            level = self.level
        self.SPG._menu_activity_userchoice(self.currentactname, level, cycles)
        
    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.logger.debug("nextlevel called with: %s" % level)
        if self.levelup:
            self.logger.debug("levelup is True, continue")
            return True
        self.level = level
        if level > self.old_level:
            self.old_level = level
        self.dbmapper = dbmapper
        self.next_activity()
        return True

    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        pass 
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        pass
    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass
        
    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        if self.levelup:
            self.levelup = False
            self.SPG.tellcore_level_end(store_db=False, \
                                    level=min(6, self.level + 1), \
                                    levelup=True)
        for event in events:
            if event.type in (MOUSEBUTTONDOWN, MOUSEMOTION):
                self.actives.update(event)
                
        return 
        



# -*- coding: utf-8 -*-

# Copyright (c) 2006 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPOptionParser.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# will becomes the thing to parse the options given at startup.
# returns a dict
import sys, os

from SPConstants import SUPPORTEDTHEMES, ACTIVITYDATADIR
from SPVersion import optversion
from optparse import OptionParser
import ConfigParser
import logging

class OParser(OptionParser):
    def __init__(self):
        usage = "usage: %prog [options]"
        OptionParser.__init__(self, usage=usage, version=optversion)
        # set some reasonable defaults
        self.set_defaults(loglevel="warning", \
            theme="childsplay", \
            no_login=False, \
            remotemysqlserver='localhost', \
            kioskmode=False, \
            adminmode=False, \
            no_sound=False, \
            no_text=False,\
            admingui=False, \
            default_language='en_US.utf8', \
            lang='', \
            fullscreen=False, \
            bigcursor=False, \
            nocursor=False, \
            no_exit_question=False, \
            nocountdown=True, \
            user='', \
            no_level_pause=False, \
            checklog = False, \
            crapplatform=False,
            moviepointer=False,\
            plainversion=False)
        # add possible options to the parser
        self.add_option("--no-login", action="store_true",
            help="Don't show login window. This will run schoolsplay in 'anonymous' mode which means that there will be no data collecting",
        dest="no_login")
        self.add_option("--kioskmode", action="store_true",
            help="Enable kioskmode. This will run schoolsplay in kioskmode which means that there will be no keyboard support.",
        dest="kioskmode")
        self.add_option("--remote-mysqlserver", action="store_true",
            help="(NOT USED YET)The ip nummer of the remote mySQL server.",
            dest="remotemysqlserver")
        self.add_option("-t", "--theme",
            help="Set the theme schoolsplay will use.",
            dest="theme")
        self.add_option("-l", "--loglevel",
            help="Set logging level. Possible values are: debug,info,warning,error,critical",
            dest="loglevel")
        self.add_option("--adminmode", action="store_true",
            help="Enable adminmode. This will run schoolsplay in adminmode which means that certain extra possibilities are enabled",
        dest="adminmode")
        self.add_option("--no-sound", action="store_true",
            help="Disable any sound. Be aware that certain activities that rely on sound won't start.",
        dest="no_sound")
        self.add_option("--no-text",action="store_true",
            help="Disable text before every level's activities.",
            dest="no_text")
        self.add_option("--language",
            help="Set the language schoolsplay should use. Be aware that not all languages are supported yet. If your language isn't supported and you want to provide language support,please contact the schoolsplay team.",
            dest="lang")
        self.add_option("--default_language",
            help="Set the default language. Only use this if you know why you want to use it.",
            dest="default_language")
        self.add_option("--fullscreen", action="store_true",
            help="Run schoolsplay in fullscreen mode.",
        dest="fullscreen")
        self.add_option("--bigcursor", action="store_true",
            help="Use a bigger (48x48) cursor.",
        dest="bigcursor")
        self.add_option("--nocursor", action="store_true",
            help="Hide the mouse cursor. This is intended for touchscreens",
        dest="nocursor")
        self.add_option("--no-exit-question", action="store_true",
            help="No 'are you sure?' questions will be asked",
            dest="noexitquestion")
        self.add_option("--no-countdown", action="store_true",
            help="No '4,3,2,1' countdown screen will displayed between levels",
            dest="nocountdown")
        self.add_option("--user",
            help="Set the username to be used for dbase actions",
            dest="user")
        self.add_option("--no-level-pause", action="store_true",
            help="Don't show graphics and don't play a sound between levels.",
            dest="no_level_pause") 
        self.add_option("--checklog", action="store_true",
            help="Check the SP log for anomalies in the logs.",
            dest="checklog") 
        self.add_option("--crapware", action="store_true",
            help="When running on wellknown crapware like the white MSI 1900 set this option and we try to work around the various OS/hardware bugs.",
            dest="crapware")
        self.add_option("--plainversion", action="store_true",
            help="Return the version number in a plain string",
            dest="plainversion")
        self.add_option("--kill-btp",
            help="If we should kill any running btp, a pid must be given. BTP is started again when childsplay quits. (ONLY available on BTP machines)",
            dest="kill_btp") 
        self.add_option("--restart-btp",
            help="If we killed any running btp, we use this to start BTP again when childsplay quits. (ONLY available on BTP machines)",
            dest="restart_btp")          
        (self.options, self.args) = self.parse_args()
        
        # in case the user starts us with a unknown theme
        if self.options.theme not in SUPPORTEDTHEMES:
            print("WARNING: theme %s unknown, using default theme 'childsplay'" % self.options.theme)
            self.options.theme = 'childsplay'
        try:
            self.parse_themerc()
        except ConfigParser.NoSectionError, info:
            print("ERROR: Error in rc file %s" % info)
            sys.exit(1)
    
    def exit(self,  * args):
        """This overrides the OptionParser exit method"""
        if args:
            print args[1]
            self.print_help()
        sys.exit(0)
        
    def get_options(self):
        return self.options

    def get_args(self):
        return self.args

    def parse_themerc(self):
        config = ConfigParser.ConfigParser()
        print config
        rc = os.path.join(ACTIVITYDATADIR, 'SPData', 'themes', self.options.theme, 'theme.rc')
        print("DEBUG: parsing rc file %s" % rc)
        config.read(rc)
        #print config.items('menubar')
        d = {}
        d['background_login'] = config.get('login','background_login')
        d['exclude_modules'] = config.get('menubar','exclude_modules')
        d['exclude_buttons'] = config.get('menubar','exclude_buttons')
        d['menubar_position'] = config.get('menubar','menubar_position')
        d['level_indicator'] = config.get('menubar','level_indicator')
        d['menubuttons']= config.get('menubar', 'menubuttons')
        d['menubartop'] = config.get('menubar', 'menubartop')
        d['menubarbottom'] = config.get('menubar', 'menubarbottom')
        d['background'] = config.get('main', 'background')
        
        d['theme'] = self.options.theme
        self.options.theme_rc = d
        
        
            

if __name__ == '__main__':
    op = OParser()
    print op.get_options()
    
    
    
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2006-2011 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           gtk_widgets.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import subprocess
import gtk


# All widgets MUST have a 'win' attribute pointing to the gtk main window

def get_image_text_box(stock_id, label_text):
    box = gtk.HBox(False, 0)
    box.set_border_width(2)

    image = gtk.Image()
    image.set_from_stock(stock_id, gtk.ICON_SIZE_BUTTON)

    label = gtk.Label(label_text)
    
    box.pack_start(image, False, False, 3)
    box.pack_start(label, False, False, 3)
    
    image.show()
    label.show()
    return box

class Base:
    """Derive from this class to get some handy stuff.
    It assumes the child has set an attribute 'win' pointing to the GTK mainwindow"""
    
    def run(self):
        gtk.main()
    
    def keypress_escape(self, widget, event) :
        if event.keyval == gtk.keysyms.Escape :
            self.quit()

    def quit(self):
        self.win.hide()
        self.win.destroy()
        gtk.main_quit()
    
    def quit_dialog(self, text):
        dlg = gtk.MessageDialog(self.win, gtk.DIALOG_DESTROY_WITH_PARENT,\
                                    gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO,\
                                     text)
        response = dlg.run()
        if response == gtk.RESPONSE_YES:
            self.quit()
        else:
            dlg.destroy()
            return True

class _Editor:
    """Internally used object.
    Wraps a textview widget and adds a few abstraction methods.
    It adds itself to the parent"""
    def __init__(self,parent,title=''):
        self.parent = parent
        
        self.frame = gtk.Frame()
        
        self.txttagtable = gtk.TextTagTable()
        self.txtbuffer = gtk.TextBuffer(table=self.txttagtable)
        
        self.tag_h = self.txtbuffer.create_tag(background='lightblue')
        
        self.txtview = gtk.TextView(buffer=self.txtbuffer)
        self.txtview.set_border_window_size(gtk.TEXT_WINDOW_LEFT, 20)                
        self.txtview.connect("expose_event", self.line_numbers_expose)
        
        self.parent.add(self.txtview)
        self.parent.show_all()
        
        self.old_start_iter = None

    def get_all_text(self):
        """Return all text from the widget"""
        startiter = self.txtbuffer.get_start_iter()
        enditer = self.txtbuffer.get_end_iter()
        txt = self.txtbuffer.get_text(startiter,enditer)
        if not txt:
            return []
        if '\n' in txt:
            txt = txt.split('\n')
        else:# assuming one line without a end of line
            txt = [txt]
        return txt
        
    def set_text(self,txt):
        """Load a text in the widget"""
        try:
            txt = ''.join(txt)
            utxt = unicode(txt)
        except Exception,info:
            print "Failed to set text in source buffer"
            print info
            return
        self.txtbuffer.set_text(utxt.rstrip('\n'))
        
    def set_highlight(self,line):
        """Highlight the line in the editor"""
        if self.old_start_iter:
            self.txtbuffer.remove_tag(self.tag_h,self.old_start_iter,self.old_end_iter)
        end_iter = self.txtbuffer.get_iter_at_line(line)
        end_iter.forward_to_line_end()  
        start_iter = self.txtbuffer.get_iter_at_line(line)
        self.txtbuffer.apply_tag(self.tag_h,start_iter,end_iter)
        self.old_start_iter,self.old_end_iter = start_iter,end_iter
    
    def reset_highlight(self):
        self.set_highlight(1)
    
    def line_numbers_expose(self, widget, event, user_data=None):
        text_view = widget
  
        # See if this expose is on the line numbers window
        left_win = text_view.get_window(gtk.TEXT_WINDOW_LEFT)
        
        if event.window == left_win:
            type = gtk.TEXT_WINDOW_LEFT
            target = left_win
        else:
            return False
        
        first_y = event.area.y
        last_y = first_y + event.area.height

        x, first_y = text_view.window_to_buffer_coords(type, 0, first_y)
        x, last_y = text_view.window_to_buffer_coords(type, 0, last_y)

        numbers = []
        pixels = []
        count = self.get_lines(first_y, last_y, pixels, numbers)
        # Draw fully internationalized numbers!
        layout = widget.create_pango_layout("")
  
        for i in range(count):
            x, pos = text_view.buffer_to_window_coords(type, 0, pixels[i])
            numbers[i] = numbers[i] + 1
            str = "%d" % numbers[i] 
            layout.set_text(str)
            widget.style.paint_layout(target, widget.state, False,
                                      None, widget, None, 2, pos + 2, layout)

        # don't stop emission, need to draw children
        return False
        
    def get_lines(self, first_y, last_y, buffer_coords, numbers):
        text_view = self.txtview
        # Get iter at first y
        iter, top = text_view.get_line_at_y(first_y)

        # For each iter, get its location and add it to the arrays.
        # Stop when we pass last_y
        count = 0
        size = 0

        while not iter.is_end():
            y, height = text_view.get_line_yrange(iter)
            buffer_coords.append(y)
            line_num = iter.get_line()
            numbers.append(line_num)
            count += 1
            if (y + height) >= last_y:
                break
            iter.forward_line()

        return count

class gtkTextEditor(Base):
    def __init__(self, parent, xmlalbum, photoindex, filelist):
        """Object that packs a GTK window with an text editor and some buttons
        After the GTK loop is left this object is still alive and you can get
        the editors text content with a call to 'get_editor_text'.
        When the user doesn't use the Save button the 'get_editor_text' will
        return an empty list.
        """
        self._editortext = []
        self._title = ''
        self.parent = parent
        self.album = xmlalbum['album']
        self.xmlalbum = xmlalbum
        self.photoindex = photoindex
        self.basepath = xmlalbum['path']
        self.filelist = filelist[:]
        self.photo_id = os.path.basename(self.filelist[self.photoindex])
        
        window = gtk.Window()
        window.set_modal(True)
        window.connect("destroy", lambda w: gtk.main_quit())
        window.connect("key-press-event", self.keypress_escape)
        #window.set_default_size(800, 600)
        window.set_geometry_hints(window, min_width=800, min_height=600, max_width=800, max_height=600)
        window.set_title("GTK window")
        
        vbox = gtk.VBox(False, 4)
        
        _text = _("You can edit the text shown below, if any. Be aware that the album name with a semicolon prepends the title. \nUse the 'Next' button to go to the next photo. \nWhen ready use the 'Close' button to return to the photo album.")
        lbl = gtk.Label()
        lbl.set_use_markup(True)
        lbl.set_markup("<b>%s</b>" % _text)
        vbox.pack_start(lbl, expand=False, padding=4)
        
        frame = gtk.Frame()
        frame.set_size_request(400,308)
        self.image_layout = gtk.Layout(None,None)
        
        frame.add(self.image_layout)
        vbox.pack_start(frame, expand=True, fill=True, padding=0)
        
        hbox = gtk.HBox(False, 2)
        lbl = gtk.Label()
        lbl.set_use_markup(True)
        lbl.set_markup("<b>%s</b>" % _("Title: "))
        hbox.pack_start(lbl, expand=False, fill=False, padding=4)
        self.entry_title = gtk.Entry()
        hbox.pack_start(self.entry_title, expand=True, fill=True, padding=4)
        vbox.pack_start(hbox, expand=True, fill=False, padding=0)
        
        frame = gtk.Frame()
        scrolledwindow = gtk.ScrolledWindow()
        scrolledwindow.set_size_request(300,150)
        scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.E = _Editor(scrolledwindow)
        
        frame.add(scrolledwindow)
        vbox.pack_start(frame, expand=True, fill=False)
        
        hbox = gtk.HBox(False, 2)
        but = gtk.Button()
        box = get_image_text_box(gtk.STOCK_STOP, _("Close"))
        but.add(box)
        but.connect("clicked", self.on_cancel_button_clicked)
        hbox.pack_start(but, expand=True, fill=False, padding=16)
        
        box = get_image_text_box(gtk.STOCK_SAVE, _("Next"))
        but = gtk.Button()
        but.add(box)
        but.connect("clicked", self.on_next_button_clicked)
        hbox.pack_start(but, expand=True, fill=False, padding=16)
        vbox.pack_start(hbox, expand=True, fill=True, padding=4)
        
        window.add(vbox)       
        window.show_all()
        
        settings = gtk.settings_get_default()
        settings.props.gtk_button_images = True

        self.win = window
        
        # set first item, the rest is set by next_image
        imgpath = os.path.join(self.basepath, self.photo_id)
        title = self.album[self.photo_id]['title']
        text = self.album[self.photo_id]['text']
        self.set_content(imgpath, title, text)
        self.photoindex += 1
    
    def on_cancel_button_clicked(self, widget):
        self.parent.save_editor(self.entry_title.get_text(), self.E.get_all_text(), self.photo_id)
        return self.quit_dialog(_("Do you want to stop this editor ? \nAll your changes are already saved."))
    
    def on_next_button_clicked(self, widget):
        self.next_image()
            
    def next_image(self):
        if self.photoindex > len(self.filelist) - 1:
            self.photoindex = 0
        self.photo_id = os.path.basename(self.filelist[self.photoindex])
        imgpath = os.path.join(self.basepath, self.photo_id)
        title = self.album[self.photo_id]['title']
        text = self.album[self.photo_id]['text']
        self.set_content(imgpath, title, text)
        self.photoindex += 1
       
        
    def set_content(self, imagepath, title, text):
        img = gtk.Image()
        pixbuf = gtk.gdk.pixbuf_new_from_file(imagepath)
        scaled_buf = pixbuf.scale_simple(400,300,gtk.gdk.INTERP_BILINEAR)
        img.set_from_pixbuf(scaled_buf)
        img.show() 
        self.image_layout.put(img,200,4)
        self.E.set_text(text)
        self.entry_title.set_text(title)
        
    
if __name__ == '__main__':
    import __builtin__
    __builtin__.__dict__['_'] = lambda x:x
    class parent:
        def save_editor(self, *args):
            print args
    t = gtkTextEditor(parent(), None)
    t.run()
            
        
        



# -*- coding: utf-8 -*-

# Copyright (c) 2006-2008 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPKeyMaps.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

# This module holds all the keymap conversion tables used to convert the key event
# into the proper non-latin character.
# Only needed in non-Latin locales.

# TODO: Check the maps. Only arabic and hebrew are probably correct.

class KeyMaps(object):
    supportedmaps = ('he', 'ar', 'el','ru')
    he={
        u'Q':u'/',
        u'W':u'\'',
        u'E':u'ק',
        u'R':u'ר',
        u'T':u'א',
        u'Y':u'ט',
        u'U':u'ו',
        u'I':u'ן',
        u'O':u'ם',
        u'P':u'פ',
        u'[':u']',
        u']':u'[',
        u'A':u'ש',
        u'S':u'ד',
        u'D':u'ג',
        u'F':u'כ',
        u'G':u'ע',
        u'H':u'י',
        u'J':u'ח',
        u'K':u'ל',
        u'L':u'ך',
        u';':u'ף',
        u'\'':u',',
        u'Z':u'ז',
        u'X':u'ס',
        u'C':u'ב',
        u'V':u'ה',
        u'B':u'נ',
        u'N':u'מ',
        u'M':u'צ',
        u',':u'ת',
        u'.':u'ץ',
        u'/':u'.'
        }

    ar={
        u'Q':u'ض',
        u'W':u'ص',
        u'E':u'ث',
        u'R':u'ق',
        u'T':u'ف',
        u'Y':u'غ',
        u'U':u'ع',
        u'I':u'ه',
        u'O':u'خ',
        u'P':u'ح',
        u'[':u'ج',
        u']':u'د',
        u'A':u'ش',
        u'S':u'س',
        u'D':u'ي',
        u'F':u'ب',
        u'G':u'ل',
        u'H':u'ا',
        u'J':u'ت',
        u'K':u'ن',
        u'L':u'م',
        u';':u'ك',
        u'\'':u'ط',
        u'Z':u'<',
        u'X':u'ئ',
        u'C':u'ء',
        u'V':u'ؤ',
        u'B':u'ر',
        u'N':u'ﻻ',
        u'M':u'ى',
        u',':u'ة',
        u'.':u'و',
        u'/':u'ز'
        }
            
    ru={
        u'Q':u'й',
        u'W':u'ц',
        u'E':u'у',
        u'R':u'к',
        u'T':u'е',
        u'Y':u'н',
        u'U':u'г',
        u'I':u'ш',
        u'O':u'щ',
        u'P':u'з',
        u'[':u'х',
        u']':u'ъ',
        u'A':u'ф',
        u'S':u'ы',
        u'D':u'в',
        u'F':u'а',
        u'G':u'п',
        u'H':u'р',
        u'J':u'о',
        u'K':u'л',
        u'L':u'д',
        u';':u'ж',
        u'\'':u'э',
        u'Z':u'я',
        u'X':u'ч',
        u'C':u'с',
        u'V':u'м',
        u'B':u'и',
        u'N':u'т',
        u'M':u'ь',
        u',':u'б',
        u'.':u'ю',
        u'/':u'.'
        }

    el = {
        u'E':u'Ε',
        u'R':u'Ρ',
        u'T':u'Τ',
        u'Y':u'Υ',
        u'U':u'Θ',
        u'I':u'Ι',
        u'O':u'Ο',
        u'P':u'Π',
        u'A':u'Α',
        u'S':u'Σ',
        u'D':u'Δ',
        u'F':u'Φ',
        u'G':u'Γ',
        u'H':u'Η',
        u'J':u'Ξ',
        u'K':u'Κ',
        u'L':u'Λ',
        u'Z':u'Ζ',
        u'X':u'Χ',
        u'C':u'Ψ',
        u'V':u'Ω',
        u'B':u'Β',
        u'N':u'Ν',
        u'M':u'Μ'
        }



# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPWidgets.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from base import *
from buttons import *
from dialogs import *
from funcs import *
from text import *

from SPConstants import NoGtk
if not NoGtk:
    from gtk_widgets import *

LOCALE_RTL = utils.get_locale_local()[1]
if LOCALE_RTL:
    ALIGN_RIGHT = True
    ALIGN_LEFT = False
else:
    ALIGN_LEFT = True
    ALIGN_RIGHT = False

class TextRectException(Exception):
    pass





# -*- coding: utf-8 -*-

# Copyright (c) 2010 Stas Zytkiewicz stas.zytkiewicz@schoolsplay.org
#
#           findit_sp.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#create logger, logger was setup in SPLogging
import logging
# In your Activity class -> 
# self.logger =  logging.getLogger("childsplay.findit_sp.Activity")
# self.logger.error("I don't understand logger")
# See SP manual for more info 

module_logger = logging.getLogger("childsplay.findit_sp")

# standard modules you probably need
import os, sys, random

import pygame
from pygame.constants import *

import utils
from SPConstants import *
import SPSpriteUtils
from SPWidgets import ImgButton, TransPrevNextButton, PrevNextButton, SimpleTransImgButton

class Img_Display(SPSpriteUtils.SPSprite):
    """Object that displays an image at a determined position and reacts to mouseclicks.
    It accepts rects for the differences and provides methods to do stuff with the rects."""
    points = 0
    wrongs = 0
    def __init__(self, image, rectlist, snd, wrongsprite, rchash):
        self.logger = logging.getLogger('schoolsplay.findit_sp.Img_Display')
        SPSpriteUtils.SPSprite.__init__(self, image)
        self.connect_callback(self.callback, MOUSEBUTTONDOWN)
        self.goodsnd, self.wrongsnd = snd        
        self.wrongsprite = wrongsprite
        theme = rchash['theme']
        self.maxwrong = int(rchash[theme]['wrongs'])
        Img_Display.wrongs = 0
        i = 0
        self.offset = int(rchash[theme]['wrong_rect_offset_%s' % rchash['level']]) * 2
        
        self.RectHash = {}
        # This will become a hash with rect style tuples as keys because pygame rects aren't hashable
        self.RectHashCollide = {}
        for r in rectlist:
            tempr = r.inflate(self.offset, self.offset)
            self.RectHash[i] = tempr
            i += 1
            self.RectHashCollide[(tempr[0], tempr[1], tempr[2], tempr[3])] = i
        pygame.draw.rect(self.image, BLACK, self.rect, 4)
        self.wrongpos_x = 375
        self.wrongpos_y = 320
                
    def show_hint(self, widget, event, data):
        k, r = random.choice(self.RectHash.items())
        Img_Display.points -= 1
        if Img_Display.points < 0:
            Img_Display.points = 0
        self.draw_diff(k, r)
        # needed as we draw on the sprite which is updated already by the parentclass 
        # and when we the last hint our parent is never updated.
        self.display_sprite()
        if not self.RectHash:
            
            return True
            
    def set_position(self, x, y):
        self.rect.move_ip(x, y)
        self.ImgOffset = (x, y)

    def register_observer(self, obs):
        self.obs = obs
    
    def notify_observer(self, k, r):
        self.obs(k, r,self.wrongpos_y)
    
    def observer(self, k, r, wrongpos_y):
        #self.logger.debug("%s.observer called" % self.name)
        self.wrongpos_y = wrongpos_y
        pygame.display.update(pygame.draw.rect(self.image, RED, r.inflate(-self.offset, -self.offset), 10))
        self.display_sprite()
        if self.RectHash.has_key(k):
            del self.RectHash[k]
        
    def callback(self,sprite,event,*args):
        #self.logger.debug("%s.callback called" % self.name)
        goodflag = False
        mouserect = pygame.Rect(event.pos[0] - self.ImgOffset[0], \
                              event.pos[1] - self.ImgOffset[1], \
                              4, 4)
        mousepos = mouserect.topleft
        colliderects = mouserect.collidedictall(self.RectHashCollide)
        templist = []
        for item in colliderects:
            r = item[0]
            i = item[1]
            d = utils.calcdist(mousepos, pygame.Rect(*r).center)
            templist.append((d, (r, i)))
        templist.sort()
        if templist:
            target = templist[0][1]
            Img_Display.points += 1
            self.draw_diff(target[1]-1, pygame.Rect(target[0]))
            goodflag = True
        if not goodflag:
            if Img_Display.wrongs*2 < self.maxwrong*2 or self.maxwrong == -1:
                Img_Display.wrongs += 1
                if self.wrongsprite:
                    self.wrongsprite.display_sprite((self.wrongpos_x, self.wrongpos_y))
                self.wrongpos_y += 50
                self.wrongsnd.play()
            else:
                return True
        if not self.RectHash:
            return True
    
    def draw_diff(self, k, r):
        self.goodsnd.play()
        self.points += 1
        pygame.display.update(pygame.draw.rect(self.image, RED, r.inflate(-self.offset, -self.offset), 10))
        self.display_sprite()
        try:
            del self.RectHash[k]
            del self.RectHashCollide[(r[0], r[1], r[2], r[3])]
        except KeyError:
            pass
        self.notify_observer(k, r)
    
    def get_result(self):
        return (Img_Display.points, Img_Display.wrongs)
    
class Activity:
    """  Base class mandatory for any SP activty.
    The activity is started by instancing this class by the core.
    This class must at least provide the following methods.
    start (self) called by the core before calling 'next_level'.
    post_next_level (self) called once by the core after 'next_level' *and* after the 321 count. 
    next_level (self,level) called by the core when the user changes levels.
    loop (self,events) called 40 times a second by the core and should be your 
                      main eventloop.
    get_helptitle (self) must return the title of the game can be localized.
    get_help (self) must return a list of strings describing the activty.
    get_helptip (self) must return a list of strings or an empty list
    get_name (self) must provide the activty name in english, not localized in lowercase.
    stop_timer (self) must stop any timers if any.
  """

    def __init__(self,SPGoodies):
        """SPGoodies is a class object that SP sets up and will contain references
        to objects, callback methods and observers
        TODO: add more explaination"""
        self.logger =  logging.getLogger("childsplay.findit_sp.Activity")
        self.logger.info("Activity started")
        self.SPG = SPGoodies
        self.lang = self.SPG.get_localesetting()[0][:2]
        self.scoredisplay = self.SPG.get_scoredisplay()
        self.theme = self.SPG.get_theme()
        self.screen = self.SPG.get_screen()
        self.screenclip = self.SPG.get_screenclip()
        self.blit_pos = self.screenclip.left, self.screenclip.top
        self.orgscreen = pygame.Surface(self.screenclip.size) # we use this to restore the screen
        self.orgscreen.blit(self.screen, (0, 0), self.screenclip)
        self.backgr = self.SPG.get_background()
        # The location of the activities Data dir
        self.my_datadir = os.path.join(self.SPG.get_libdir_path(),'CPData','Findit_spData')
        self.rchash = utils.read_rcfile(os.path.join(self.my_datadir, 'findit.rc'))
        self.rchash['theme'] = self.theme
        self.logger.debug("found rc: %s" % self.rchash)
        # Location of the CPData dir which holds some stuff used by multiple activities
        self.CPdatadir = os.path.join(self.SPG.get_libdir_path(),'CPData')
        self.DbAssets = os.path.join(self.SPG.get_libdir_path(),'CPData', 'DbaseAssets')
        # Location of the alphabet sounds dir
        self.absdir = self.SPG.get_absdir_path()# alphabet sounds dir
        # You MUST call SPInit BEFORE using any of the SpriteUtils stuff
        # it returns a reference to the special CPGroup
        self.actives = SPSpriteUtils.SPInit(self.screen,self.backgr)
        self.actives.set_onematch(True)
    
    def clear_screen(self):
        self.screen.blit(self.orgscreen,self.blit_pos)
        self.backgr.blit(self.orgscreen,self.blit_pos)
        pygame.display.update()
        for s in self.actives:
            s.display_sprite()

    def get_moviepath(self):
        movie = os.path.join(self.my_datadir,'help.avi')
        return movie

    def refresh_sprites(self):
        """Mandatory method, called by the core when the screen is used for blitting
        and the possibility exists that your sprites are affected by it."""
        self.actives.redraw()

    def get_helptitle(self):
        """Mandatory method"""
        return _("Findit")
    
    def get_name(self):
        """Mandatory method, returnt string must be in lowercase."""
        return "findit_sp"
    
    def get_help(self):
        """Mandatory methods"""
        text = [_("The aim of this activity:"),
                _("On the screen you will see two almost identical pictures.\nHowever, there are differences between the images."), 
                " ", 
                _("When you spot a difference, touch it with your finger.\nYou can click the question mark in the middle of the screen if you need a hint."), 
        " ",
        _("The arrows to the left take you to the previous image.")
        ]
        return text 
    
    def get_helptip(self):
        """Mandatory method, when no tips available returns an empty string"""
        return ''
        
    def get_helptype(self):
        """Mandatory method, you must set an type"""
        # Possible types are: Memory, Math, Puzzle, Keyboardtraining, Mousetraining
        #                     Language, Alphabet, Fun, Miscellaneous
        return _("Miscellaneous")
    
    
    def get_helplevels(self):
        """Mandatory method, must return a string with the number of levels
        in the follwing format:
        _("This level has %s levels") % number-of-levels"""
        return _("This activity has %s levels") % 6

    def start(self):
        """Mandatory method."""
        self.GoodSound=utils.load_sound(os.path.join(self.CPdatadir, 'good.ogg'))
        self.WrongSound=utils.load_sound(os.path.join(self.CPdatadir, 'wrong.ogg'))
        p = os.path.join(self.CPdatadir,'good_%s.png' % self.lang)
        if not os.path.exists(p):
            p = os.path.join(self.CPdatadir,'thumbs.png')
        self.ThumbsUp = SPSpriteUtils.MySprite(utils.load_image(p))

        i = utils.load_image(os.path.join(self.my_datadir, 'hint.png'))
        i_ro = utils.load_image(os.path.join(self.my_datadir, 'hint_ro.png'))
        self.HintBut = SimpleTransImgButton(i,i_ro, (370, 200))
        if int(self.rchash[self.theme]['show_errors']):
            self.wrongImg = SPSpriteUtils.MySprite(utils.load_image(os.path.join(self.my_datadir, 'incorrect.png')))
        else:
            self.wrongImg = None
        prev = os.path.join(self.my_datadir, 'findit_prev.png')
        prev_ro = os.path.join(self.my_datadir, 'findit_prev_ro.png')
        next = os.path.join(self.my_datadir, 'findit_next.png')
        next_ro = os.path.join(self.my_datadir, 'findit_next_ro.png')
        self.prevnextBut = TransPrevNextButton((370, 460), \
                                              self._cbf_prevnext_button, \
                                              prev, prev_ro, next, next_ro)
        self.imgdir = os.path.join(self.my_datadir, 'images', self.theme)
        if not os.path.exists(self.imgdir):
            self.imgdir = os.path.join(self.my_datadir, 'images','default')                                      
        self.score = 0
        self.AreWeDT = False
        # get language code
        loclang = utils.get_locale_local()[0]
        
    def dailytraining_pre_level(self, level):
        """Mandatory method"""
        pass
        
    def dailytraining_next_level(self, level, dbmapper):
        """Mandatory method.
        This should handle the DT logic"""
        self.AreWeDT = True
        self.SPG.tellcore_disable_level_indicator()
        self.next_level(level, dbmapper)
        return True

    def pre_level(self, level):
        """Mandatory method"""
        pass

    def next_level(self,level,dbmapper):
        """Mandatory method.
        Return True if there levels left.
        False when no more levels left."""
        self.logger.debug("next_level called with %s" % level)
        if level > 6: return False # We only have 5 levels
        self.rchash['level'] = level
        self.levelupcount = 1
        self.done = 0
        self.dbscore = 0
        # db_mapper is a Python class object that provides easy access to the 
        # dbase.
        self.dbmapper = dbmapper
        # TODO: are the number of differences the same for each level ? We now have 3 hardcoded
        self.dbmapper.insert('total_diffs', int(self.rchash[self.theme]['exercises'])*3)
        self.totalwrongs = 0
        self.level = level
        self.actives.empty()
        self.clear_screen()
        self.previous_screen = None
        if self.blit_pos[1] == 0:
            y = 10
        else:
            y = 110
        Img_Display.wrongs = 0
        Img_Display.points = 0
        self.logger.debug("searching images in %s"  % self.imgdir)
        p = os.path.join(self.imgdir, 'img_diffs_%s.csv')
        self.ImgDiffHash = {}
        self.ImgDiffList = []
        try:
            lines = open(p % level, 'r').readlines()
        except IOError, info:
            raise utils.MyError, info
        for line in lines:
            k, v = line[:-1].split(';',1)
            try:
                self.ImgDiffHash[os.path.join(self.imgdir, k)] = \
                                [pygame.Rect(eval(x,{'__builtins__': None},\
                                {'True':True,'False':False})) for x in v.split(':')]
            except SyntaxError, info:
                self.logger.error('Badly formed rect line in %s; %s' % (p % level, line))
        self.LevelScoreHash = {1:3, 2:3, 3:3, 4:3, 5:3, 6:3}
        
        self.start_exercise()
        return True
        
    def post_next_level(self):
        """Mandatory method.
        This is called once by the core after 'next_level' *and* after the 321 count.
        You should place stuff in here that run in a seperate thread like sound play."""
        
    def get_score(self,timespend):
        """Mandatory method.
        @timespend is the time spend inside the level as it's calculated
        by the core.
        returns the score value for the past level.
        return None if no score value is used"""
        m,s = timespend.split(':')
        seconds = int(m)*60 + int(s)
        # M = Maximum score (number of differences * total images)
        # P = Users result in points
        # F = Fraction of total result related to points. (maximum result is 10)
        # F1 = Fraction of result related to time
        # C = multiplier
        # S = time spend in seconds
        M = float(3 * int(self.rchash[self.theme]['exercises']) * self.level)
        P = float(self.dbscore)
        F = 7.0
        F1 = 3.0
        C = 0.2
        S = float(seconds)
        points = max((F/100) * (P/(M / 100)), 1.0)
        time = min(F1 / ((S / (M * 2)) ** C), F1)
        self.logger.info("@scoredata@ %s level %s M %s P %s S %s points %s time %s" %\
                          (self.get_name(),self.level, M, P, S, points, time))
        result = points + time
        return result
    
    def stop_timer(self):
        """You *must* provide a method to stop timers if you use them.
        The SPC will call this just in case and catch the exception in case 
        there's no 'stop_timer' method""" 
        try:
            self.timer.stop()
        except:
            pass

    def loop(self,events):
        """Mandatory method.
        This is the main eventloop called by the core 30 times a minute."""
        for event in events:
            if event.type in (MOUSEBUTTONDOWN, MOUSEBUTTONUP, MOUSEMOTION):
                if self.actives.update(event):
                    self.end_exercise()
                    pygame.event.clear()
                    break
        return 
        
    def _cbf_prevnext_button(self, sprite, event, data):
        self.logger.debug('_cbf_prevnext_button called with: %s' % data[0])
        if data[0] == 'prev': 
            self._cbf_prev_button(data)
        else: 
            self._cbf_next_button(data)
    
    def _cbf_next_button(self, data):
        self.logger.debug("_cbf_next_button: %s" % data)
        if not self.previous_screen:
            return
        self.clear_screen()
        self.actives.add(self.currentlist)
        for s in self.currentlist:
            s.display_sprite()
        self.prevnextBut.toggle()
        self.HintBut.display_sprite()
        self.actives.add(self.HintBut)
          
    def _cbf_prev_button(self, data):
        self.logger.debug("_cbf_prev_button: %s" % data)
        self.actives.remove(self.currentlist)
        self.HintBut.erase_sprite()
        self.actives.remove(self.HintBut)
        pygame.display.update(self.screen.blit(self.previous_screen, (0, self.blit_pos[1])))
        self.prevnextBut.toggle()
        
    def end_exercise(self, thumbs=True): 
        
        result = self.ImgA.get_result()
        
        self.logger.debug("got score from ImgA, points: %s, wrongs: %s" % (result[0], result[1]))
        self.totalwrongs += result[1]
        self.dbscore = (result[0] - result[1]) * self.level
        self.score += self.dbscore * 10
        self.scoredisplay.set_score(self.score)
        # TODO: we have hardcoded a check for the amount of wrongs iso rc file values.
        if self.totalwrongs < 4:
            levelup = 1
        else:
            levelup = 0
        self.actives.empty()
        if self.currentlist and not self.AreWeDT:
            self.previous_screen = pygame.Surface((800, 500))
            surf = pygame.display.get_surface()
            self.previous_screen.blit(surf, (0, 0), (0, self.blit_pos[1], 800, 500))
        utils.sleep(2000)
        if not result[0] == self.LevelScoreHash[self.level]:
            thumbs = False
            self.clear_screen()
        if thumbs:
            self.clear_screen()
            self.ThumbsUp.display_sprite((229, 120))
            utils.sleep(2000)
            self.ThumbsUp.erase_sprite()
        if not self.ImgDiffHash or self.done == int(self.rchash[self.theme]['exercises']):
            self.dbmapper.insert('wrongs', self.totalwrongs)
            if self.AreWeDT:
                self.SPG.tellcore_level_end(level=self.level)
            else:
                self.SPG.tellcore_level_end(store_db=True,
                                            level=min(6, self.level),
                                            levelup=levelup)
        else:
            self.start_exercise()
      
    def start_exercise(self):
        if self.blit_pos[1] == 0:
            y = 10
        else:
            y = 110
        
        k = random.choice(self.ImgDiffHash.keys())
        
        v = self.ImgDiffHash[k]
        del self.ImgDiffHash[k]
        ImgA = Img_Display(utils.load_image(k+'A.jpg'), v,\
                                     (self.GoodSound, self.WrongSound), self.wrongImg, self.rchash)
        ImgA.set_position(15, y)
        ImgA.name = 'ImgA'# needed for debugging only
        ImgB = Img_Display(utils.load_image(k+'B.jpg'), v,\
                                 (self.GoodSound, self.WrongSound), self.wrongImg, self.rchash)
        ImgB.set_position(435, y)
        ImgB.name = 'ImgB'
        ImgA.register_observer(ImgB.observer)
        ImgB.register_observer(ImgA.observer)
        self.currentlist = [ImgA, ImgB]
        self.HintBut.connect_callback(ImgA.show_hint, MOUSEBUTTONUP, self)
        self.HintBut.display_sprite()
        #self.prevnextBut.enable(False)
        self.actives.add(self.HintBut)
        self.actives.add([ImgA, ImgB])
        if self.previous_screen and not self.AreWeDT:
            self.actives.add(self.prevnextBut.get_actives())
            self.prevnextBut.enable(True)
        self.actives.redraw()
        self.ImgA, self.ImgB = ImgA, ImgB
        self.done += 1
    
    



# -*- coding: utf-8 -*-

# Copyright (c) 2007-2010 Stas Zykiewicz <stas.zytkiewicz@schoolsplay.org>
#
#           SPDataManagerCreateDbase.py
# This program is free software; you can redistribute it and/or
# modify it under the terms of version 3 of the GNU General Public License
# as published by the Free Software Foundation.  A copy of this license should
# be included in the file GPL-3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#create logger, logger was configured in SPLogging
import os

import logging
module_logger = logging.getLogger("childsplay.SPDataManagerCreateDbase")
from SPConstants import ACTIVITYDATADIR, DBASE, HOMEDIR, WHICHDBASE, CONTENTDBASE

from utils import MyError, read_rcfile
try:
    import sqlalchemy as sqla
    import sqlalchemy.orm as sqlorm
except ImportError:
    module_logger.exception("No sqlalchemy package found")
    raise MyError
else:
    if sqla.__version__ < '0.5':
        module_logger.error("Found sqlalchemy version %s" % sqla.__version__)
        module_logger.error("Your version of sqlalchemy is to old, please upgrade to version >= 0.4")
        raise MyError
    module_logger.debug("using sqlalchemy %s" % sqla.__version__)
try:
    import sqlalchemy.exceptions as sqlae
except ImportError:
    from sqlalchemy import exc as sqlae
import SQLTables
import SPORMs

# determine which dbase to use, we support mySQL and SQLite
USEMYSQL = False
USESQLITE = False
rc_hash = ''

if WHICHDBASE == 'mysql':
    try:
        import MySQLdb, _mysql_exceptions
    except ImportError:
        module_logger.exception("No MySQLdb package found which is needed by sqlalchemy")
        module_logger.error("The WHICHDBASE constant is set in SPConstants to 'mysql'")
        raise MyError
    else:
        module_logger.info("Using mySQL for the user dbase")
        USEMYSQL = True
        # suppress warnings from MySQLdb which seems to use Python warnings iso exceptions
        import warnings
        warnings.filterwarnings("ignore")
        try:
            import db_conf as dbrc
            rckind = 'db_conf'
        except ImportError:
            import db_dev as dbrc
            rckind = 'db_dev'
        rc_hash = dbrc.rc
        if rc_hash['default']['production']:
            kind = 'production'
        else:
            kind = 'develop'
        rc_hash['kind'] = kind
        rc_hash['path'] = rckind
else:
    module_logger.info("Using SQLite for the user dbase")
    USESQLITE = True

class DbaseMaker:
    def __init__(self, theme, debug_sql=False):
        self.logger = logging.getLogger("childsplay.SPDataManagerCreateDbase.DbaseMaker")
        self.logger.debug("Starting")
        self.usersdbasepath= os.path.join(HOMEDIR, theme, DBASE)
        self.contentdbasepath = os.path.join(HOMEDIR, theme, CONTENTDBASE)
        # We could also use a mysql dbase, local or remote
        try:
            ##### start userdb stuff ########
            if USESQLITE:
                self.logger.debug("Starting SQLite users dbase, %s" % self.usersdbasepath)
                engine = sqla.create_engine('sqlite:///%s' % self.usersdbasepath)
            elif USEMYSQL:
                self.logger.info("Starting mySQL dbase, %s" % rc_hash['sp_users']['dbasename'])
                import MySQLdb, _mysql_exceptions
                self.logger.info("Using conffile %s" % rc_hash['path'])
                db=MySQLdb.connect(host=rc_hash['sp_users']['host'], \
                                   user=rc_hash['sp_users']['user'], \
                                   passwd=rc_hash['sp_users']['user_pass'])
                c = db.cursor()
                c.execute('''CREATE DATABASE IF NOT EXISTS %s''' % rc_hash['sp_users']['dbasename'])
                
                db.close()
                url = 'mysql://%s:%s@%s/%s' %\
                                    (rc_hash['sp_users']['user'], \
                                    rc_hash['sp_users']['user_pass'], \
                                    rc_hash['sp_users']['host'], \
                                    rc_hash['sp_users']['dbasename'])
                engine = sqla.create_engine(url)
            else:
                self.logger.error("I'm confused about which dbase to use, please check your settings in SPConstants and make sure you have all the dependencies installed")
                raise MyError
            # The rest is the same for all dbases thanks to sqlalchemy  
            self.metadata_usersdb = sqla.MetaData(engine)
            self.metadata_usersdb.bind.echo = debug_sql
            self.sqltb = SQLTables.SqlTables(self.metadata_usersdb)
            # returns lookup table to get table objects by name and creates tables
            self.orms_userdb = self.sqltb._create_userdb_tables(self.metadata_usersdb)
            self.user_engine = engine
            self.tables = self.sqltb.tableslist
            #self._check_tables_uptodate()
            #### end userdb stuff#####
            #### start contentdb stuff #####
            if USESQLITE:
                self.logger.debug("Starting SQLite content dbase, %s" % self.contentdbasepath)
                engine = sqla.create_engine('sqlite:///%s' % self.contentdbasepath)
            elif USEMYSQL:
                engine = sqla.create_engine('mysql://%s:%s@%s/%s' %\
                                        (rc_hash['btp_content']['user'], \
                                        rc_hash['btp_content']['user_pass'], \
                                        rc_hash['btp_content']['host'], \
                                        rc_hash['btp_content']['dbasename']))
                
            self.metadata_contentdb = sqla.MetaData(engine)
            self.metadata_contentdb.bind.echo = debug_sql
            self.orms_content_db = SQLTables.create_contentdb_orms(self.metadata_contentdb)
            self.content_engine = engine
            ##############################
            self.all_orms = {}
            self.all_orms.update(self.orms_userdb)
            self.all_orms.update(self.orms_content_db)
        except (AttributeError, sqlae.SQLAlchemyError), info:
            self.logger.exception("Failed to start the DBase, %s" % info)
            raise MyError, info
            
    def get_engines(self):
        return(self.content_engine, self.user_engine)
    def get_all_orms(self):
        return self.all_orms
    def get_orms(self):
        return (self.orms_content_db, self.orms_userdb)
    def get_metadatas(self):
        return (self.metadata_contentdb, self.metadata_usersdb)
    def _get_sqltables(self):    
        return self.sqltb
        
            
